/* tarot implements the rules of the tarot game
 * Copyright (C) 2017, 2018, 2019  Vivien Kraus
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "gettext.h"
#include <tarot/features_private.h>
#include "xalloc.h"
#include "gettext.h"
#define _(String) dgettext (PACKAGE, String)

static int libtarot_initialized = 0;

char *tarot_libtarot_program_name = PACKAGE_NAME;

static char *libtarot_localedir = NULL;
static char *libtarot_datadir = NULL;
static char *libtarot_pkgdatadir = NULL;

static void
tarot_set_localedir (const char *localedir)
{
  free (libtarot_localedir);
  libtarot_localedir = strdup (localedir);
}

void
tarot_set_datadir (const char *datadir)
{
  free (libtarot_datadir);
  free (libtarot_pkgdatadir);
  libtarot_datadir = strdup (datadir);
  libtarot_pkgdatadir = xmalloc (strlen (libtarot_datadir)
                                 + strlen ("/") + strlen (PACKAGE) + 1);
  strcpy (libtarot_pkgdatadir, libtarot_datadir);
  strcat (libtarot_pkgdatadir, "/");
  strcat (libtarot_pkgdatadir, PACKAGE);
}

int
tarot_init (const char *localedir)
{
  assert (libtarot_initialized == 0);
  tarot_set_localedir (localedir);
  bindtextdomain (PACKAGE, libtarot_localedir);
  _tarot_features_init (libtarot_pkgdatadir);
  libtarot_initialized = 1;
  return 0;
}

void
tarot_quit (void)
{
  libtarot_initialized = 0;
}

const char *
tarot_libtarot_version (void)
{
  static const char *effective_version = LIBTAROT_VERSION_INFO;
  return effective_version;
}

#include <tarot/features_private_impl.h>
